import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'package:get/get.dart';
import 'package:encrypt_shared_preferences/provider.dart';

import 'package:f0ckapp/service/media_service.dart';
import 'package:f0ckapp/controller/localization_controller.dart';
import 'package:f0ckapp/utils/appversion.dart';
import 'package:f0ckapp/controller/theme_controller.dart';
import 'package:f0ckapp/controller/media_controller.dart';
import 'package:f0ckapp/screens/detail_view.dart';
import 'package:f0ckapp/screens/media_grid.dart';
import 'package:f0ckapp/controller/auth_controller.dart';

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  await SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
  await EncryptedSharedPreferencesAsync.initialize('VokTnbAbemBUa2j9');
  await MyTranslations.loadTranslations();
  await AppVersion.init();
  Get.put(MediaService());
  Get.put(MediaController());
  Get.put(AuthController());
  LocalizationController localizationController = Get.put(LocalizationController());
  final ThemeController themeController = Get.put(ThemeController());

  Get.addTranslations(MyTranslations.instance.keys);
  Get.locale = localizationController.currentLocale.value;

  //Locale systemLocale = WidgetsBinding.instance.platformDispatcher.locale;

  runApp(
    Obx(
      () => MaterialApp(
        locale: Get.locale,
        navigatorKey: Get.key,
        theme: themeController.currentTheme.value,
        debugShowCheckedModeBanner: false,
        onGenerateRoute: (RouteSettings settings) {
          final uri = Uri.parse(settings.name ?? '/');

          if (uri.path == '/' || uri.pathSegments.isEmpty) {
            return MaterialPageRoute(builder: (_) => MediaGrid());
          }

          if (uri.pathSegments.length == 1) {
            final int id = int.parse(uri.pathSegments.first);
            return MaterialPageRoute(builder: (_) => DetailView(initialId: id));
          }

          return MaterialPageRoute(builder: (_) => MediaGrid());
        },
      ),
    ),
  );
}
